@@ -28,11 +28,14 @@
 #include "zend_hash.h"
 #include "zend_extensions.h"
 #include "ext/standard/info.h"
+#include "ext/standard/php_rand.h"
 #include "php_suhosin.h"
 #include "zend_compile.h"
 #include "zend_llist.h"
 #include "SAPI.h"
 
+#include "sha256.h"
+
 
 static void (*old_execute)(zend_op_array *op_array TSRMLS_DC);
 static void suhosin_execute(zend_op_array *op_array TSRMLS_DC);
@@ -764,7 +767,11 @@ int ih_mail(IH_HANDLER_PARAMS)
 
 int ih_querycheck(IH_HANDLER_PARAMS)
 {
+#ifdef PHP_ATLEAST_5_3
+    void **p = zend_vm_stack_top(TSRMLS_C) - 1;
+#else
 	void **p = EG(argument_stack).top_element-2;
+#endif
 	unsigned long arg_count;
 	zval **arg;
 	char *query, *s, *e;
@@ -920,7 +927,11 @@ int ih_querycheck(IH_HANDLER_PARAMS)
 
 int ih_fixusername(IH_HANDLER_PARAMS)
 {
+#ifdef PHP_ATLEAST_5_3
+    void **p = zend_vm_stack_top(TSRMLS_C) - 1;
+#else
 	void **p = EG(argument_stack).top_element-2;
+#endif
 	unsigned long arg_count;
 	zval **arg;char *prefix, *postfix, *user;
 	zval *backup, *my_user;
@@ -1089,6 +1100,368 @@ static int ih_function_exists(IH_HANDLER_PARAMS)
 	return (1);
 }
 
+/* MT RAND FUNCTIONS */
+
+/*
+	The following php_mt_...() functions are based on a C++ class MTRand by
+	Richard J. Wagner. For more information see the web page at
+	http://www-personal.engin.umich.edu/~wagnerr/MersenneTwister.html
+
+	Mersenne Twister random number generator -- a C++ class MTRand
+	Based on code by Makoto Matsumoto, Takuji Nishimura, and Shawn Cokus
+	Richard J. Wagner  v1.0  15 May 2003  rjwagner@writeme.com
+
+	The Mersenne Twister is an algorithm for generating random numbers.  It
+	was designed with consideration of the flaws in various other generators.
+	The period, 2^19937-1, and the order of equidistribution, 623 dimensions,
+	are far greater.  The generator is also fast; it avoids multiplication and
+	division, and it benefits from caches and pipelines.  For more information
+	see the inventors' web page at http://www.math.keio.ac.jp/~matumoto/emt.html
+
+	Reference
+	M. Matsumoto and T. Nishimura, "Mersenne Twister: A 623-Dimensionally
+	Equidistributed Uniform Pseudo-Random Number Generator", ACM Transactions on
+	Modeling and Computer Simulation, Vol. 8, No. 1, January 1998, pp 3-30.
+
+	Copyright (C) 1997 - 2002, Makoto Matsumoto and Takuji Nishimura,
+	Copyright (C) 2000 - 2003, Richard J. Wagner
+	All rights reserved.                          
+
+	Redistribution and use in source and binary forms, with or without
+	modification, are permitted provided that the following conditions
+	are met:
+
+	1. Redistributions of source code must retain the above copyright
+	   notice, this list of conditions and the following disclaimer.
+
+	2. Redistributions in binary form must reproduce the above copyright
+	   notice, this list of conditions and the following disclaimer in the
+	   documentation and/or other materials provided with the distribution.
+
+	3. The names of its contributors may not be used to endorse or promote 
+	   products derived from this software without specific prior written 
+	   permission.
+
+	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+	LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+	A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
+	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+	PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+	LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+	NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+	The original code included the following notice:
+
+	When you use this, send an email to: matumoto@math.keio.ac.jp
+    with an appropriate reference to your work.
+
+	It would be nice to CC: rjwagner@writeme.com and Cokus@math.washington.edu
+	when you write.
+*/
+
+#define N             624                 /* length of state vector */
+#define M             (397)                /* a period parameter */
+#define hiBit(u)      ((u) & 0x80000000U)  /* mask all but highest   bit of u */
+#define loBit(u)      ((u) & 0x00000001U)  /* mask all but lowest    bit of u */
+#define loBits(u)     ((u) & 0x7FFFFFFFU)  /* mask     the highest   bit of u */
+#define mixBits(u, v) (hiBit(u)|loBits(v)) /* move hi bit of u to hi bit of v */
+
+#define twist(m,u,v)  (m ^ (mixBits(u,v)>>1) ^ ((php_uint32)(-(php_int32)(loBit(u))) & 0x9908b0dfU))
+
+/* {{{ php_mt_initialize
+ */
+static inline void suhosin_mt_initialize(php_uint32 seed, php_uint32 *state)
+{
+	/* Initialize generator state with seed
+	   See Knuth TAOCP Vol 2, 3rd Ed, p.106 for multiplier.
+	   In previous versions, most significant bits (MSBs) of the seed affect
+	   only MSBs of the state array.  Modified 9 Jan 2002 by Makoto Matsumoto. */
+
+	register php_uint32 *s = state;
+	register php_uint32 *r = state;
+	register int i = 1;
+
+	*s++ = seed & 0xffffffffU;
+	for( ; i < N; ++i ) {
+		*s++ = ( 1812433253U * ( *r ^ (*r >> 30) ) + i ) & 0xffffffffU;
+		r++;
+	}
+}
+/* }}} */
+
+static inline void suhosin_mt_init_by_array(php_uint32 *key, int keylen, php_uint32 *state)
+{
+    int i, j, k;
+    suhosin_mt_initialize(19650218U, state);
+    i = 1; j = 0;
+    k = (N > keylen ? N : keylen);
+    for (; k; k--) {
+        state[i] = (state[i] ^ ((state[i-1] ^ (state[i-1] >> 30)) * 1664525U)) + key[j] + j;
+        i++; j = (j+1) % keylen;
+        if (i >= N) { state[0] = state[N-1]; i=1; }
+    }
+    for (k=N-1; k; k--) {
+        state[i] = (state[i] ^ ((state[i-1] ^ (state[i-1] >> 30)) * 1566083941U)) - i;
+        i++;
+        if (i >= N) { state[0] = state[N-1]; i=1; }
+    }
+    state[0] = 0x80000000U;
+}
+/* }}} */
+
+
+/* {{{ suhosin_mt_reload
+ */
+static inline void suhosin_mt_reload(php_uint32 *state, php_uint32 **next, int *left)
+{
+	/* Generate N new values in state
+	   Made clearer and faster by Matthew Bellew (matthew.bellew@home.com) */
+
+	register php_uint32 *p = state;
+	register int i;
+
+	for (i = N - M; i--; ++p)
+		*p = twist(p[M], p[0], p[1]);
+	for (i = M; --i; ++p)
+		*p = twist(p[M-N], p[0], p[1]);
+	*p = twist(p[M-N], p[0], state[0]);
+	*left = N;
+	*next = state;
+}
+/* }}} */
+
+/* {{{ suhosin_mt_srand
+ */
+static void suhosin_mt_srand(php_uint32 seed TSRMLS_DC)
+{
+	/* Seed the generator with a simple uint32 */
+	suhosin_mt_initialize(seed, SUHOSIN_G(mt_state));
+	suhosin_mt_reload(SUHOSIN_G(mt_state), &SUHOSIN_G(mt_next), &SUHOSIN_G(mt_left));
+
+	/* Seed only once */
+	SUHOSIN_G(mt_is_seeded) = 1;
+}
+/* }}} */
+
+/* {{{ suhosin_mt_rand
+ */
+static php_uint32 suhosin_mt_rand(TSRMLS_D)
+{
+	/* Pull a 32-bit integer from the generator state
+	   Every other access function simply transforms the numbers extracted here */
+	
+	register php_uint32 s1;
+
+	if (SUHOSIN_G(mt_left) == 0) {
+    	suhosin_mt_reload(SUHOSIN_G(mt_state), &SUHOSIN_G(mt_next), &SUHOSIN_G(mt_left));
+	}
+	--SUHOSIN_G(mt_left);
+		
+	s1 = *SUHOSIN_G(mt_next)++;
+	s1 ^= (s1 >> 11);
+	s1 ^= (s1 <<  7) & 0x9d2c5680U;
+	s1 ^= (s1 << 15) & 0xefc60000U;
+	return ( s1 ^ (s1 >> 18) );
+}
+/* }}} */
+
+/* {{{ suhosin_gen_entropy
+ */
+static void suhosin_gen_entropy(php_uint32 *seedbuf TSRMLS_DC)
+{
+    /* On a modern OS code, stack and heap base are randomized */
+    unsigned long code_value  = (unsigned long)suhosin_gen_entropy;
+    unsigned long stack_value = (unsigned long)&code_value;
+    unsigned long heap_value  = (unsigned long)SUHOSIN_G(r_state);
+    suhosin_SHA256_CTX   context;
+    
+    code_value ^= code_value >> 32;
+    stack_value ^= stack_value >> 32;
+    heap_value ^= heap_value >> 32;
+    
+    seedbuf[0] = code_value;
+    seedbuf[1] = stack_value;
+    seedbuf[2] = heap_value;
+    seedbuf[3] = time(0);
+#ifdef PHP_WIN32
+    seedbuf[4] = GetCurrentProcessId();
+#else
+    seedbuf[4] = getpid();
+#endif
+    seedbuf[5] = (php_uint32) 0x7fffffff * php_combined_lcg(TSRMLS_C);
+    
+    suhosin_SHA256Init(&context);
+	suhosin_SHA256Update(&context, (void *) seedbuf, sizeof(php_uint32) * 6);
+	suhosin_SHA256Final(seedbuf, &context);
+}
+/* }}} */
+
+
+/* {{{ suhosin_srand_auto
+ */
+static void suhosin_srand_auto(TSRMLS_D)
+{
+    php_uint32 seed[8];    
+    suhosin_gen_entropy(&seed[0] TSRMLS_CC);
+
+	suhosin_mt_init_by_array(seed, 8, SUHOSIN_G(r_state));
+	suhosin_mt_reload(SUHOSIN_G(r_state), &SUHOSIN_G(r_next), &SUHOSIN_G(r_left));
+
+	/* Seed only once */
+	SUHOSIN_G(r_is_seeded) = 1;
+}
+/* }}} */
+
+/* {{{ suhosin_mt_srand_auto
+ */
+static void suhosin_mt_srand_auto(TSRMLS_D)
+{
+    php_uint32 seed[8];    
+    suhosin_gen_entropy(&seed[0] TSRMLS_CC);
+
+	suhosin_mt_init_by_array(seed, 8, SUHOSIN_G(mt_state));
+	suhosin_mt_reload(SUHOSIN_G(mt_state), &SUHOSIN_G(mt_next), &SUHOSIN_G(mt_left));
+
+	/* Seed only once */
+	SUHOSIN_G(mt_is_seeded) = 1;
+}
+/* }}} */
+
+
+/* {{{ suhosin_srand
+ */
+static void suhosin_srand(php_uint32 seed TSRMLS_DC)
+{
+	/* Seed the generator with a simple uint32 */
+	suhosin_mt_initialize(seed+0x12345, SUHOSIN_G(r_state));
+	suhosin_mt_reload(SUHOSIN_G(r_state), &SUHOSIN_G(r_next), &SUHOSIN_G(r_left));
+
+	/* Seed only once */
+	SUHOSIN_G(r_is_seeded) = 1;
+}
+/* }}} */
+
+/* {{{ suhosin_mt_rand
+ */
+static php_uint32 suhosin_rand(TSRMLS_D)
+{
+	/* Pull a 32-bit integer from the generator state
+	   Every other access function simply transforms the numbers extracted here */
+	
+	register php_uint32 s1;
+
+	if (SUHOSIN_G(r_left) == 0) {
+    	suhosin_mt_reload(SUHOSIN_G(r_state), &SUHOSIN_G(r_next), &SUHOSIN_G(r_left));
+	}
+	--SUHOSIN_G(r_left);
+		
+	s1 = *SUHOSIN_G(r_next)++;
+	s1 ^= (s1 >> 11);
+	s1 ^= (s1 <<  7) & 0x9d2c5680U;
+	s1 ^= (s1 << 15) & 0xefc60000U;
+	return ( s1 ^ (s1 >> 18) );
+}
+/* }}} */
+
+static int ih_srand(IH_HANDLER_PARAMS)
+{
+    int argc = ZEND_NUM_ARGS();
+	long seed;
+
+	if (zend_parse_parameters(argc TSRMLS_CC, "|l", &seed) == FAILURE || SUHOSIN_G(srand_ignore)) {
+        RETVAL_FALSE;
+    	return (1);
+    }
+
+    if (argc == 0) {
+        suhosin_srand_auto(TSRMLS_C);
+    } else {
+        suhosin_srand(seed TSRMLS_CC);
+    }
+	RETVAL_TRUE;
+	return (1);
+}
+
+static int ih_mt_srand(IH_HANDLER_PARAMS)
+{
+    int argc = ZEND_NUM_ARGS();
+	long seed;
+
+	if (zend_parse_parameters(argc TSRMLS_CC, "|l", &seed) == FAILURE || SUHOSIN_G(mt_srand_ignore)) {
+        RETVAL_FALSE;
+    	return (1);
+    }
+    
+    if (argc == 0) {
+        suhosin_mt_srand_auto(TSRMLS_C);
+    } else {
+        suhosin_mt_srand(seed TSRMLS_CC);
+    }
+	RETVAL_TRUE;
+	return (1);
+}
+
+static int ih_mt_rand(IH_HANDLER_PARAMS)
+{
+    int argc = ZEND_NUM_ARGS();
+    long min;
+	long max;
+	long number;
+
+	if (argc != 0 && zend_parse_parameters(argc TSRMLS_CC, "ll", &min, &max) == FAILURE) {
+	    return (1);        
+	}
+
+	if (!SUHOSIN_G(mt_is_seeded)) {
+		suhosin_mt_srand_auto(TSRMLS_C);
+	}
+
+	number = (long) (suhosin_mt_rand(TSRMLS_C) >> 1);
+	if (argc == 2) {
+		RAND_RANGE(number, min, max, PHP_MT_RAND_MAX);
+	}
+
+	RETURN_LONG(number);
+}
+
+static int ih_rand(IH_HANDLER_PARAMS)
+{
+    int argc = ZEND_NUM_ARGS();
+    long min;
+	long max;
+	long number;
+
+	if (argc != 0 && zend_parse_parameters(argc TSRMLS_CC, "ll", &min, &max) == FAILURE) {
+	    return (1);        
+	}
+
+	if (!SUHOSIN_G(r_is_seeded)) {
+		suhosin_srand_auto(TSRMLS_C);
+	}
+
+	number = (long) (suhosin_rand(TSRMLS_C) >> 1);
+	if (argc == 2) {
+		RAND_RANGE(number, min, max, PHP_MT_RAND_MAX);
+	}
+
+	RETURN_LONG(number);
+}
+
+static int ih_getrandmax(IH_HANDLER_PARAMS)
+{
+    int argc = ZEND_NUM_ARGS();
+
+    if (argc != 0) {
+		ZEND_WRONG_PARAM_COUNT();
+    }
+    
+	RETVAL_LONG(PHP_MT_RAND_MAX);
+	return (1);
+}
 
 internal_function_handler ihandlers[] = {
     { "preg_replace", ih_preg_replace, NULL, NULL, NULL },
@@ -1096,6 +1469,13 @@ internal_function_handler ihandlers[] = {
     { "symlink", ih_symlink, NULL, NULL, NULL },
     { "phpinfo", ih_phpinfo, NULL, NULL, NULL },
 	
+	{ "srand", ih_srand, NULL, NULL, NULL },
+	{ "mt_srand", ih_mt_srand, NULL, NULL, NULL },
+	{ "rand", ih_rand, NULL, NULL, NULL },
+	{ "mt_rand", ih_mt_rand, NULL, NULL, NULL },
+	{ "getrandmax", ih_getrandmax, NULL, NULL, NULL },
+	{ "mt_getrandmax", ih_getrandmax, NULL, NULL, NULL },
+	
     { "ocilogon", ih_fixusername, (void *)1, NULL, NULL },
     { "ociplogon", ih_fixusername, (void *)1, NULL, NULL },
     { "ocinlogon", ih_fixusername, (void *)1, NULL, NULL },
@@ -1148,13 +1528,31 @@ internal_function_handler ihandlers[] = {
 static void suhosin_execute_internal(zend_execute_data *execute_data_ptr, int return_value_used TSRMLS_DC)
 {
 	char *lcname;
-	int function_name_strlen;
+	int function_name_strlen, free_lcname = 0;
 	zval *return_value;
+	zend_class_entry *ce = NULL;
 	int ht;
 	internal_function_handler *ih;
-	
+
+#ifdef ZEND_ENGINE_2
+	ce = ((zend_internal_function *) execute_data_ptr->function_state.function)->scope;
+#endif
 	lcname = ((zend_internal_function *) execute_data_ptr->function_state.function)->function_name;
 	function_name_strlen = strlen(lcname);
+	
+	/* handle methodcalls correctly */
+	if (ce != NULL) {
+		char *tmp = (char *) emalloc(function_name_strlen + 2 + ce->name_length + 1);
+		memcpy(tmp, ce->name, ce->name_length);
+		memcpy(tmp+ce->name_length, "::", 2);
+		memcpy(tmp+ce->name_length+2, lcname, function_name_strlen);
+		lcname = tmp;
+		free_lcname = 1;
+		function_name_strlen += ce->name_length + 2;
+		lcname[function_name_strlen] = 0;
+		zend_str_tolower(lcname, function_name_strlen);
+	}
+	
 #ifdef ZEND_ENGINE_2  
 	return_value = (*(temp_variable *)((char *) execute_data_ptr->Ts + execute_data_ptr->opline->result.u.var)).var.ptr;
 #else
@@ -1168,30 +1566,26 @@ static void suhosin_execute_internal(zend_execute_data *execute_data_ptr, int re
 	
 		if (SUHOSIN_G(eval_whitelist) != NULL) {
 			if (!zend_hash_exists(SUHOSIN_G(eval_whitelist), lcname, function_name_strlen+1)) {
-			    suhosin_log(S_EXECUTOR, "function outside of eval whitelist called: %s()", lcname);
-			    FUNCTION_WARNING()
-			    suhosin_bailout(TSRMLS_C);
+				suhosin_log(S_EXECUTOR, "function outside of eval whitelist called: %s()", lcname);
+				goto execute_internal_bailout;
 			}
 		} else if (SUHOSIN_G(eval_blacklist) != NULL) {
 			if (zend_hash_exists(SUHOSIN_G(eval_blacklist), lcname, function_name_strlen+1)) {
-			    suhosin_log(S_EXECUTOR, "function within eval blacklist called: %s()", lcname);
-			    FUNCTION_WARNING()
-			    suhosin_bailout(TSRMLS_C);
+				suhosin_log(S_EXECUTOR, "function within eval blacklist called: %s()", lcname);
+				goto execute_internal_bailout;
 			}
 		}
 	}
 	
 	if (SUHOSIN_G(func_whitelist) != NULL) {
 		if (!zend_hash_exists(SUHOSIN_G(func_whitelist), lcname, function_name_strlen+1)) {
-		    suhosin_log(S_EXECUTOR, "function outside of whitelist called: %s()", lcname);
-		    FUNCTION_WARNING()
-		    suhosin_bailout(TSRMLS_C);
+			suhosin_log(S_EXECUTOR, "function outside of whitelist called: %s()", lcname);
+			goto execute_internal_bailout;
 		}
 	} else if (SUHOSIN_G(func_blacklist) != NULL) {
 		if (zend_hash_exists(SUHOSIN_G(func_blacklist), lcname, function_name_strlen+1)) {
-		    suhosin_log(S_EXECUTOR, "function within blacklist called: %s()", lcname);
-		    FUNCTION_WARNING()
-		    suhosin_bailout(TSRMLS_C);
+			suhosin_log(S_EXECUTOR, "function within blacklist called: %s()", lcname);
+			goto execute_internal_bailout;
 		}
 	}
 	
@@ -1210,6 +1604,16 @@ static void suhosin_execute_internal(zend_execute_data *execute_data_ptr, int re
 	} else {
 		old_execute_internal(execute_data_ptr, return_value_used TSRMLS_CC);
 	}
+	if (free_lcname == 1) {
+		efree(lcname);
+	}
+	return;
+execute_internal_bailout:
+	if (free_lcname == 1) {
+		efree(lcname);
+	}
+	FUNCTION_WARNING()
+	suhosin_bailout(TSRMLS_C);
 }
 /* }}} */
 
